
; 8-Servo controller driver for MSX-BASIC (trough printer port) ver 1.0
;
; Made By: NYYRIKKI 2013
;
; Usage:
;
; Select first servo number and then servo angle
; USR0: Set servo number (X = 0-7)
; USR1: Set servo angle 1-255 (0 = Turn servo control off)
; USR2: INIT / EXIT (Z=INIT, NZ=EXIT)
; USR3: CHANGE CPU MODE (0-2) (MSX tR only)
;
; Notes:
; - Test compilation has been done on Sjasm v0.42b8
; - This works only if you have 50Hz display & Standard Z80 or R800 speed
; - Value 128 should put servo near center
; - There is a tiny difference in Z80 and R800 timing, hope you don't mind :-)
;

        DB #FE
        DW BEGIN
        DW END
        DW START

        ORG #C100

;------------------------------------------

USR:    EQU #F7F8
USR0:   EQU #F39A
HOOK:   EQU #FD9A
TABLE:  EQU #C000

OUTPORT: EQU #91

BEGIN:

        DB 0 ; Last send out byte

SERVO:  DB 0

USRS:   DW SETSERV, VECTOR, EXIT, SETCPU

LOCATION:
        DW #C6,#CE,#D6,#DE,#E6,#EE,#F6,#FE
START:
        DI

        LD A,(#FFE8)
        OR 2
	OUT (#99),A
	LD (#FFE8),A
	LD A,#89
	OUT (#99),A

        LD HL,TABLE
        LD DE,TABLE+1
        LD BC,#FF
        LD (HL),0
        LDIR

        LD B,8
        LD HL,LOCATION+1
.LOOP
        LD (HL),0
        INC HL
        INC HL
        DJNZ .LOOP

        CALL ISACTIVE
        RET Z


        LD HL,USRS
        LD DE,USR0
        LD BC,8
        LDIR

        LD HL,HOOK
        LD DE,OLDHOOK
        LD BC,5
        LDIR
        LD HL,NEWINT
        LD A,(#2D)
        CP 3
        JP NZ,.NOTR
        CALL #183
.INITHOOK:
        AND A
        JR Z,.NOTR
        LD HL,NEWINTR
.NOTR
        DI
        LD A,#C3
        LD (HOOK),A
        LD (HOOK+1),HL
        RET
        
SETCPU:
        LD A,(#2D)
        CP 3
        RET NZ
        DI
        LD A,(USR)
        AND 127
        CP 3
        RET NC
        XOR 128
        CALL #180
        DI
        CALL ISACTIVE
        RET NZ
        LD A,(USR)
        LD HL,NEWINT
        JP START.INITHOOK

EXIT:
        DI
        LD A,(USR)
        AND A
        JP Z,START
        CALL ISACTIVE
        RET NZ
        LD HL,OLDHOOK
        LD DE,HOOK
        LD BC,5
        LDIR
        RET


ISACTIVE:
        LD HL,(HOOK+1)
        LD DE,NEWINT
        RST #20
        RET Z
        LD DE,NEWINTR
        RST #20
        RET

SETSERV:
        LD A,(USR)
        AND 7
        LD (SERVO),A
        RET

VECTOR:
        LD A,(SERVO)
        ADD A,A
        LD E,A
        LD D,0
        LD HL,LOCATION
        ADD HL,DE
        LD A,(HL)
        LD (BITON),A
        XOR 64
        LD (BITOFF),A
        INC HL
        LD E,(HL)
        LD A,(USR)
        LD (HL),A
        LD H,TABLE/256
        LD L,E
        CP L
        JR C,SMALLER

LOOP1:
        CP L
        RET Z
        SET 0,(HL)
BITON:  EQU $-1
        INC L
        JP LOOP1

SMALLER:

LOOP2:
        DEC L
        RES 0,(HL)
BITOFF: EQU $-1
        CP L
        JP NZ,LOOP2
        RET

NEWINT:
        PUSH AF
        DI
        LD HL,TABLE
        LD BC,OUTPORT+218*256      ; 0.85ms delay
        OUTI
.LOOP   DJNZ .LOOP   ;14/9
[255]   OUTI         ;18
        POP AF
OLDHOOK:
[6]     RET


NEWINTR:
        PUSH AF
        DI
        LD HL,TABLE
        PUSH HL
        POP IX
        LD BC,OUTPORT+ 224*256      ; 0.85ms delay
        OUTI
.LOOP   EX (SP),HL
        EX (SP),HL
        ADC A,(ix+1)
        DJNZ .LOOP
.LOOP2
        OUTI
        EX (SP),HL     ;5
        EX (SP),HL     ;5
        LD A,B         ;1
        DEC A          ;2
        JR NZ,.LOOP2   ;3
        POP AF
        JP OLDHOOK
END: